@@ -11,6 +11,7 @@ import weakref
 
 from pyqode.qt import QtCore, QtWidgets, QtGui
 from pyqode.core.api import utils
+from pyqode.core import icons
 from pyqode.core.dialogs import DlgUnsavedFiles
 from pyqode.core._forms import popup_open_files_ui
 from .tab_bar import TabBar
@@ -19,6 +20,8 @@ from .code_edits import GenericCodeEdit, TextCodeEdit
 from pyqode.core._forms import pyqode_core_rc
 assert pyqode_core_rc
 
+DEFAULT_EXTENSION = '.txt'
+
 
 def _logger():
     return logging.getLogger(__name__)
@@ -41,7 +44,34 @@ class DraggableTabBar(TabBar):
         self.setAcceptDrops(True)
         self.setMouseTracking(True)
         self.setElideMode(QtCore.Qt.ElideNone)
-
+        if self.parent().plus_button:
+            self._plus_button = QtWidgets.QPushButton(
+                icons.icon(qta_name='fa.plus-circle'),
+                None,
+                self.parent()
+            )
+            self._plus_button.setFlat(True)
+            self._plus_button.show()
+            self._plus_button.clicked.connect(self._on_plus_button_clicked)
+            self.parent().last_tab_closed.connect(self._move_plus_button)
+            self.paintEvent = self._custom_paint_event
+        
+    def _move_plus_button(self):
+        self._plus_button.move(
+            self.mapTo(self.parent(), QtCore.QPoint(
+                max(0, self.geometry().right()),
+                max(0, (self.height() - self._plus_button.height()) // 2)
+            ))
+        )
+        
+    def _on_plus_button_clicked(self):
+        self.parent().parent().create_new_document()
+        
+    def _custom_paint_event(self, event):
+        """Is set as paintEvent when the plus button is shown."""
+        super(DraggableTabBar, self).paintEvent(event)
+        self._move_plus_button()
+        
     def mousePressEvent(self, event):
         if event.button() == QtCore.Qt.LeftButton:
             self._pos = event.pos()  # _pos is a QPoint defined in the header
@@ -130,28 +160,38 @@ class BaseTabWidget(QtWidgets.QTabWidget):
 
     _detached_window_class = None
 
-    def __init__(self, parent, tab_bar_shortcuts=True):
+    def __init__(
+        self,
+        parent,
+        tabs_movable=True,
+        plus_button=True,
+        tab_context_menu=True,
+        empty_context_menu=True,
+    ):
         super(BaseTabWidget, self).__init__(parent)
+        self.plus_button = plus_button
         self._current = None
         self.currentChanged.connect(self._on_current_changed)
         self.tabCloseRequested.connect(self._on_tab_close_requested)
-
         tab_bar = DraggableTabBar(self)
-        tab_bar.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
-        tab_bar.customContextMenuRequested.connect(self._show_tab_context_menu)
-        tab_bar.tab_move_request.connect(self._on_tab_move_request)
+        if tab_context_menu:
+            tab_bar.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
+            tab_bar.customContextMenuRequested.connect(
+                self._show_tab_context_menu
+            )
+        if tabs_movable:
+            tab_bar.tab_move_request.connect(self._on_tab_move_request)
         self.setTabBar(tab_bar)
         self.setAcceptDrops(True)
         self.setUsesScrollButtons(True)
 
         #: A list of additional context menu actions
-        self._tab_bar_shortcuts = tab_bar_shortcuts
         self.context_actions = []
         self.a_close = None
         self.a_close_all = None
         self._menu_pos = None
+        self._n_pinned = 0  # the number of pinned tabs
         self._create_tab_bar_menu()
-
         self.detached_tabs = []
 
     def tab_under_menu(self):
@@ -169,13 +209,57 @@ class BaseTabWidget(QtWidgets.QTabWidget):
         Closes the active editor
         """
         self.tabCloseRequested.emit(self.tab_under_menu())
+        
+    def is_pinned(self, index):
+        """Checks whether the tab at the gives is pinned."""
+        return index < self._n_pinned
+    
+    def _unpinned_range(self):
+        """Gets a range of tabs starting from the first unpinned tab."""
+        return range(self._n_pinned, self.count())
+        
+    def pin(self, tab=None):
+        """Pins a tab. If no tab is specified, then the tab under the last
+        context menu is used.
+        """
+        if tab:
+            i = self.indexOf(tab)
+        else:
+            i = self.tab_under_menu()
+            tab = self.widget(i)
+        if self.is_pinned(i):
+            return
+        tab.unpinned_icon = self.tabIcon(i)
+        tab.unpinned_text = self.tabText(i)
+        self.setTabText(i, None)
+        self.setTabIcon(i, icons.icon(qta_name='fa.lock'))
+        self._on_tab_move_request(tab, 0)
+        self._n_pinned += 1
+    
+    def unpin(self, tab=None):
+        """Unpins a tab. If no tab is specified, then the tab under the last
+        context menu is used.
+        """
+        if tab:
+            i = self.indexOf(tab)
+        else:
+            i = self.tab_under_menu()
+            tab = self.widget(i)
+        if not self.is_pinned(i):
+            return
+        self.setTabIcon(i, tab.unpinned_icon)
+        self.setTabText(i, tab.unpinned_text)
+        self._n_pinned -= 1
 
     def close_others(self):
         """
         Closes every editors tabs except the current one.
         """
         current_widget = self.widget(self.tab_under_menu())
-        if self._try_close_dirty_tabs(exept=current_widget):
+        if self._try_close_dirty_tabs(
+            exept=current_widget,
+            tab_range=self._unpinned_range()
+        ):
             i = 0
             while self.count() > 1:
                 widget = self.widget(i)
@@ -190,11 +274,13 @@ class BaseTabWidget(QtWidgets.QTabWidget):
         """
         current_widget = self.widget(self.tab_under_menu())
         index = self.indexOf(current_widget)
-        if self._try_close_dirty_tabs(tab_range=range(index)):
-            while True:
-                widget = self.widget(0)
+        if self._try_close_dirty_tabs(
+            tab_range=range(self._n_pinned, index),
+        ):
+            while self.count() > self._n_pinned:
+                widget = self.widget(self._n_pinned)
                 if widget != current_widget:
-                    self.remove_tab(0)
+                    self.remove_tab(self._n_pinned)
                 else:
                     break
 
@@ -204,7 +290,11 @@ class BaseTabWidget(QtWidgets.QTabWidget):
         """
         current_widget = self.widget(self.tab_under_menu())
         index = self.indexOf(current_widget)
-        if self._try_close_dirty_tabs(tab_range=range(index + 1, self.count())):
+        index = max(index, self._n_pinned)
+        current_widget = self.widget(index)
+        if self._try_close_dirty_tabs(
+            tab_range = range(index + 1, self.count())
+        ):
             while True:
                 widget = self.widget(self.count() - 1)
                 if widget != current_widget:
@@ -216,11 +306,11 @@ class BaseTabWidget(QtWidgets.QTabWidget):
         """
         Closes all editors
         """
-        if self._try_close_dirty_tabs():
-            while self.count():
-                widget = self.widget(0)
+        if self._try_close_dirty_tabs(tab_range=self._unpinned_range()):
+            while self.count() > self._n_pinned:
+                widget = self.widget(self._n_pinned)
                 self.tab_closed.emit(widget)
-                self.remove_tab(0)
+                self.remove_tab(self._n_pinned)
             return True
         return False
 
@@ -300,46 +390,57 @@ class BaseTabWidget(QtWidgets.QTabWidget):
         return True
 
     def _create_tab_bar_menu(self):
-        context_mnu = QtWidgets.QMenu()
-        for name, slot, icon in [
-                (_('Close tab'), self.close, 'document-close'),
-                (_('Close tabs to the left'), self.close_left, 'tab-close-other'),
-                (_('Close tabs to the right'), self.close_right, 'tab-close-other'),
-                (_('Close others tabs'), self.close_others, 'tab-close-other'),
-                (_('Close all tabs'), self.close_all,
-                 'project-development-close-all'),
+        tab = self.widget(self.tab_under_menu())
+        index = self.indexOf(tab)
+        context_mnu = QtWidgets.QMenu(self)
+        if self.is_pinned(index):
+            pin_action = QtWidgets.QAction(_('Unpin'), self)
+            pin_action.triggered.connect(self.unpin)
+            pin_action.setIcon(icons.icon(qta_name='fa.unlock'))
+        else:
+            pin_action = QtWidgets.QAction(_('Pin'), self)
+            pin_action.triggered.connect(self.pin)
+            pin_action.setIcon(icons.icon(qta_name='fa.lock'))
+        context_mnu.addAction(pin_action)
+        if not self.is_pinned(index):
+            for name, slot, icon in [
                 (None, None, None),
-                (_('Detach tab'), self.detach_tab, 'tab-detach')]:
-            if name is None and slot is None:
-                qaction = QtWidgets.QAction(self)
-                qaction.setSeparator(True)
-            else:
-                qaction = QtWidgets.QAction(name, self)
-                qaction.triggered.connect(slot)
-                if icon:
-                    qaction.setIcon(QtGui.QIcon.fromTheme(icon))
-            if slot == self.close and self.a_close is None:
-                self.a_close = qaction
-                self.addAction(self.a_close)
-            elif slot == self.close_left:
-                self.a_close_left = qaction
-            elif slot == self.close_right:
-                self.a_close_right = qaction
-            elif slot == self.close_others:
-                self.a_close_others = qaction
-            elif slot == self.close_all:
-                self.a_close_all = qaction
-                self.addAction(self.a_close_all)
-            context_mnu.addAction(qaction)
-            # self.addAction(qaction)
+                (_('Close tab'), self.close, 'fa.close'),
+                (_('Close tabs to the left'), self.close_left, 'fa.close'),
+                (_('Close tabs to the right'), self.close_right, 'fa.close'),
+                (_('Close other tabs'), self.close_others, 'fa.close'),
+                (_('Close all tabs'), self.close_all, 'fa.close'),
+                (None, None, None),
+            ]:
+                if name is None and slot is None:
+                    qaction = QtWidgets.QAction(self)
+                    qaction.setSeparator(True)
+                else:
+                    qaction = QtWidgets.QAction(name, self)
+                    qaction.triggered.connect(slot)
+                    if icon:
+                        qaction.setIcon(icons.icon(qta_name=icon))
+                if slot == self.close and self.a_close is None:
+                    self.a_close = qaction
+                    self.addAction(self.a_close)
+                elif slot == self.close_left:
+                    self.a_close_left = qaction
+                elif slot == self.close_right:
+                    self.a_close_right = qaction
+                elif slot == self.close_others:
+                    self.a_close_others = qaction
+                elif slot == self.close_all:
+                    self.a_close_all = qaction
+                    self.addAction(self.a_close_all)
+                context_mnu.addAction(qaction)
         context_mnu.addSeparator()
         menu = QtWidgets.QMenu(_('Split'), context_mnu)
-        menu.setIcon(QtGui.QIcon.fromTheme('split'))
+        menu.setIcon(icons.icon(qta_name='fa.th'))
         a = menu.addAction(_('Split horizontally'))
         a.triggered.connect(self._on_split_requested)
-        a.setIcon(QtGui.QIcon.fromTheme('view-split-left-right'))
+        a.setIcon(icons.icon(qta_name='fa.arrows-h'))
         a = menu.addAction(_('Split vertically'))
-        a.setIcon(QtGui.QIcon.fromTheme('view-split-top-bottom'))
+        a.setIcon(icons.icon(qta_name='fa.arrows-v'))
         a.triggered.connect(self._on_split_requested)
         context_mnu.addMenu(menu)
         context_mnu.addSeparator()
@@ -347,19 +448,11 @@ class BaseTabWidget(QtWidgets.QTabWidget):
             context_mnu.addSeparator()
         for action in self.context_actions:
             context_mnu.addAction(action)
-
-        tab = self.widget(self.tab_under_menu())
-        index = self.indexOf(tab)
-        self.a_close_right.setVisible(0 <= index < self.count() - 1)
-        self.a_close_left.setVisible(0 < index <= self.count() - 1)
-        self.a_close_others.setVisible(self.count() > 1)
-        self.a_close_all.setVisible(self.count() > 1)
-        if self._tab_bar_shortcuts:
-            # These should not be set when multiple tabs will be visible in a
-            # splitter, because then they will become ambiguous and hence
-            # ineffective.
-            self.a_close.setShortcut('Ctrl+W')
-            self.a_close_all.setShortcut('Ctrl+Shift+W')
+        if not self.is_pinned(index):
+            self.a_close_right.setVisible(0 <= index < self.count() - 1)
+            self.a_close_left.setVisible(0 < index <= self.count() - 1)
+            self.a_close_others.setVisible(self.count() > 1)
+            self.a_close_all.setVisible(self.count() > 1)
         self._context_mnu = context_mnu
         return context_mnu
 
@@ -426,6 +519,9 @@ class BaseTabWidget(QtWidgets.QTabWidget):
 
     def _on_tab_close_requested(self, index):
         widget = self.widget(index)
+        if self.is_pinned(index):
+            self.unpin(widget)
+            return
         dirty = False
         try:
             if widget.original is None and not widget.clones:
@@ -550,6 +646,10 @@ class BaseTabWidget(QtWidgets.QTabWidget):
     def _on_tab_move_request(self, widget, new_index):
         parent = widget.parent_tab_widget
         index = parent.indexOf(widget)
+        if self.is_pinned(index):
+            new_index = min(self._n_pinned - 1, new_index)
+        else:
+            new_index = max(self._n_pinned, new_index)
         text = parent.tabText(index)
         icon = parent.tabIcon(index)
         parent.removeTab(index)
@@ -735,13 +835,22 @@ class SplittableTabWidget(QtWidgets.QSplitter):
         for splitter in self.child_splitters:
             splitter.tab_bar_visible = visible
 
-    def __init__(self, parent=None, root=True, create_popup=True,
-                 qsettings=None):
+    def __init__(
+        self,
+        parent=None,
+        root=True,
+        create_popup=False,
+        qsettings=None,
+        tabs_movable=True,
+        plus_button=True,
+        tab_context_menu=True,
+        empty_context_menu=True,
+    ):
         super(SplittableTabWidget, self).__init__(parent)
         SplittableTabWidget.tab_widget_klass._detached_window_class = \
             SplittableTabWidget.detached_window_klass
-        if root:
-            self._tab_bar_visible = True
+        self._tab_bar_visible = True
+        if root and create_popup:
             self._action_popup = QtWidgets.QAction(self)
             self._action_popup.setShortcutContext(QtCore.Qt.WindowShortcut)
             self._shortcut = 'Ctrl+T'
@@ -755,7 +864,10 @@ class SplittableTabWidget(QtWidgets.QSplitter):
         self.child_splitters = []
         self.main_tab_widget = self.tab_widget_klass(
             self,
-            tab_bar_shortcuts=False
+            tabs_movable=tabs_movable,
+            plus_button=plus_button,
+            tab_context_menu=tab_context_menu,
+            empty_context_menu=empty_context_menu
         )
         self.main_tab_widget.last_tab_closed.connect(
             self._on_last_tab_closed)
@@ -868,6 +980,7 @@ class SplittableTabWidget(QtWidgets.QSplitter):
         :param index: the index of the new splitter, or None to append
         :return: the new splitter
         """
+        self.main_tab_widget.unpin(widget)
         if widget.original:
             base = widget.original
         else:
@@ -1041,6 +1154,45 @@ class CodeEditTabWidget(BaseTabWidget):
     """
     default_directory = os.path.expanduser('~')
     dirty_changed = QtCore.Signal(bool)
+    
+    def __init__(
+        self,
+        parent,
+        tabs_movable=True,
+        plus_button=True,
+        tab_context_menu=True,
+        empty_context_menu=True,
+    ):
+        super(CodeEditTabWidget, self).__init__(
+            parent,
+            tabs_movable=tabs_movable,
+            plus_button=plus_button,
+            tab_context_menu=tab_context_menu,
+            empty_context_menu=empty_context_menu
+        )
+        if empty_context_menu:
+            self.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
+            self.customContextMenuRequested.connect(
+                self._empty_space_context_menu
+            )
+        
+    def _empty_space_context_menu(self, pos):
+        
+        menu = QtWidgets.QMenu(self)
+        menu.addAction(
+            icons.icon(qta_name='fa.plus-circle'),
+            _('New document'),
+            self.parent().create_new_document
+        )
+        closed_tabs_menu = self.parent().get_root_splitter().closed_tabs_menu
+        if len(closed_tabs_menu.actions()) > 0:
+            menu.addMenu(closed_tabs_menu)
+        menu.addAction(
+            icons.icon(qta_name='fa.close'),
+            _('Close all tabs'),
+            self.parent().close_all
+        )
+        menu.exec_(self.mapToGlobal(pos))
 
     @classmethod
     @utils.memoized
@@ -1218,24 +1370,53 @@ class SplittableCodeEditTabWidget(SplittableTabWidget):
 
     CLOSED_TABS_HISTORY_LIMIT = 10
 
-    def __init__(self, parent=None, root=True, qsettings=None):
+    def __init__(
+        self,
+        parent=None,
+        root=True,
+        create_popup=False,
+        qsettings=None,
+        tabs_movable=True,
+        plus_button=True,
+        tab_context_menu=True,
+        empty_context_menu=True,
+        corner_widget=False
+    ):
         SplittableTabWidget.detached_window_klass = DetachedEditorWindow
+        self._corner_widget = corner_widget
         super(SplittableCodeEditTabWidget, self).__init__(
-            parent, root, qsettings=qsettings)
+            parent,
+            root,
+            create_popup=create_popup,
+            qsettings=qsettings,
+            tabs_movable=tabs_movable,
+            plus_button=plus_button,
+            tab_context_menu=tab_context_menu,
+            empty_context_menu=empty_context_menu
+        )
         self.main_tab_widget.tabBar().double_clicked.connect(
             self.tab_bar_double_clicked.emit)
-        if root:
-            self.closed_tabs_history_btn = QtWidgets.QToolButton()
-            self.closed_tabs_history_btn.setAutoRaise(True)
-            self.closed_tabs_history_btn.setIcon(QtGui.QIcon.fromTheme(
-                'user-trash', QtGui.QIcon(':/pyqode-icons/rc/edit-trash.png')))
-            self.closed_tabs_history_btn.setPopupMode(
-                QtWidgets.QToolButton.InstantPopup)
-            self.closed_tabs_menu = QtWidgets.QMenu()
-            self.closed_tabs_history_btn.setMenu(self.closed_tabs_menu)
-            self.closed_tabs_history_btn.setDisabled(True)
-            self.main_tab_widget.setCornerWidget(self.closed_tabs_history_btn)
-            self.main_tab_widget.tab_closed.connect(self._on_tab_closed)
+        if not root:
+            return 
+        # The menu with closed tabs is used by the empty-space context menu
+        # as well as the corner widget, so we always initialize it.
+        self.closed_tabs_menu = QtWidgets.QMenu(
+            _('Re-open closed document'),
+            self
+        )
+        self.closed_tabs_menu.setIcon(icons.icon(qta_name='fa.undo'))
+        self.main_tab_widget.tab_closed.connect(self._on_tab_closed)
+        if not self._corner_widget:
+            return
+        self.closed_tabs_history_btn = QtWidgets.QToolButton()
+        self.closed_tabs_history_btn.setAutoRaise(True)
+        self.closed_tabs_history_btn.setIcon(QtGui.QIcon.fromTheme(
+            'user-trash', QtGui.QIcon(':/pyqode-icons/rc/edit-trash.png')))
+        self.closed_tabs_history_btn.setPopupMode(
+            QtWidgets.QToolButton.InstantPopup)
+        self.closed_tabs_history_btn.setMenu(self.closed_tabs_menu)
+        self.closed_tabs_history_btn.setDisabled(True)
+        self.main_tab_widget.setCornerWidget(self.closed_tabs_history_btn)
 
     @classmethod
     def register_code_edit(cls, code_edit_class):
@@ -1393,7 +1574,7 @@ class SplittableCodeEditTabWidget(SplittableTabWidget):
         return editor
 
     def create_new_document(self, base_name='New Document',
-                            extension='.txt', preferred_eol=0,
+                            extension=None, preferred_eol=0,
                             autodetect_eol=True, **kwargs):
         """
         Creates a new document.
@@ -1412,6 +1593,8 @@ class SplittableCodeEditTabWidget(SplittableTabWidget):
             widget constructor.
         :return: Code editor widget instance.
         """
+        if extension is None:
+            extension = self.default_extension()
         SplittableCodeEditTabWidget._new_count += 1
         name = '%s%d%s' % (base_name, self._new_count, extension)
         tab = self._create_code_edit(
@@ -1535,8 +1718,10 @@ class SplittableCodeEditTabWidget(SplittableTabWidget):
                         if action.toolTip() == original_path:
                             self.closed_tabs_menu.removeAction(action)
                             break
-                    self.closed_tabs_history_btn.setEnabled(
-                        len(self.closed_tabs_menu.actions()) > 0)
+                    if self._corner_widget:
+                        self.closed_tabs_history_btn.setEnabled(
+                            len(self.closed_tabs_menu.actions()) > 0
+                        )
                 return tab
 
     def close_document(self, path):
@@ -1670,7 +1855,8 @@ class SplittableCodeEditTabWidget(SplittableTabWidget):
                 action.setToolTip(path)
                 action.triggered.connect(self._open_closed_path)
                 action.setData(open_params)
-                self.closed_tabs_history_btn.setEnabled(True)
+                if self._corner_widget:
+                    self.closed_tabs_history_btn.setEnabled(True)
                 nb_actions = len(self.closed_tabs_menu.actions())
                 while nb_actions > self.CLOSED_TABS_HISTORY_LIMIT:
                     self.closed_tabs_menu.removeAction(
@@ -1693,5 +1879,13 @@ class SplittableCodeEditTabWidget(SplittableTabWidget):
             show_whitespaces=open_parameters['show_whitespaces'],
             **open_parameters['kwargs'])
         self.closed_tabs_menu.removeAction(action)
-        self.closed_tabs_history_btn.setEnabled(
-            len(self.closed_tabs_menu.actions()) > 0)
+        if self._corner_widget:
+            self.closed_tabs_history_btn.setEnabled(
+                len(self.closed_tabs_menu.actions()) > 0
+            )
+
+    def default_extension(self):
+        
+        if self.root:
+            return DEFAULT_EXTENSION
+        return self.get_root_splitter().default_extension()
